@@ -24,7 +24,8 @@
  * \brief This module contains ECMAScript implementations of various projection (isometric, orthographic) tools.
  */
 
-include("../Modify/Transform.js");
+include("scripts/Modify/Explode/Explode.js");
+include("scripts/Modify/Transform.js");
 include("scripts/ShapeAlgorithms.js");
 
 /**
@@ -229,11 +230,18 @@ Projection.prototype.postTransform = function(entity) {
 /**
  * Callback function for Transform.getOperation. Generic for all projections.
  * \param k Ignored
+ * \param rec Recursive call (don't call preTransform to move / rotate)
  */
-Projection.prototype.transform = function(entity, k, op, preview, flags) {
-    var i;
+Projection.prototype.transform = function(entity, k, op, preview, flags, rec) {
+    if (isNull(rec)) {
+        rec = false;
+    }
 
-    this.preTransform(entity);
+    var i, shapes;
+
+    if (rec!==true) {
+        this.preTransform(entity);
+    }
 
     if (this.retainTexts && isTextBasedEntity(entity)) {
         // don't explode text, only project position:
@@ -251,8 +259,23 @@ Projection.prototype.transform = function(entity, k, op, preview, flags) {
         return;
     }
 
-    // shapes that represent this entity:
-    var shapes = entity.getShapes();
+    // explode block reference to keep attributes (colors, etc.):
+    if (isBlockReferenceEntity(entity)) {
+        var explodedEntities = Explode.explodeEntity(entity);
+        for (i=0; i<explodedEntities.length; i++) {
+            var explodedEntity = explodedEntities[i];
+
+            if (isEntity(explodedEntity)) {
+                this.transform(explodedEntity, k, op, preview, flags, true);
+            }
+            else {
+                qWarning("shape in exploded block");
+            }
+        }
+        return;
+    }
+
+    shapes = entity.getShapes();
     this.addTransformedShapes(entity, shapes, op, preview, flags);
 
     // text data that is part of this entity (dimension label):
@@ -292,7 +315,9 @@ Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview
                     debugger;
                 }
 
-                s[n].simplify(0.01);
+                if (isPolylineShape(s[n])) {
+                    s[n].simplify(0.001);
+                }
 
                 hd.addBoundary(s[n], true);
             }
@@ -315,6 +340,10 @@ Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview
     }
 
     for (i=0; i<shapes.length; i++) {
+        if (isNull(shapes[i])) {
+            continue;
+        }
+
         s = this.projectShape(shapes[i].data(), preview);
         for (n=0; n<s.length; n++) {
             if (isNull(s[n])) {
@@ -372,12 +401,15 @@ Projection.prototype.isFlip = function() {
  *
  * \return Array of new projected shapes.
  */
-Projection.prototype.projectShape = function(shape, preview, trim) {
+Projection.prototype.projectShape = function(shape, preview, trim, rec) {
     var p, p2, i, k, s, ret;
 
     if (isNull(trim)) {
         trim = true;
     }
+    if (isNull(rec)) {
+        rec = false;
+    }
 
     // trim shape and project resulting segments:
     if (trim===true && (isPointShape(shape) || isLineShape(shape) || isArcShape(shape))) {
@@ -413,10 +445,13 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             }
 
             var polyline = ShapeAlgorithms.lineOrArcToPolyline(shape, numSegments);
-            var projectedPl = this.projectShape(polyline, preview, trim)[0];
-            if (!isNull(projectedPl)) {
-                projectedPl.simplify(0.01);
-                ret.push(projectedPl);
+            var projectedPls = this.projectShape(polyline, preview, trim, true);
+            if (projectedPls.length>0) {
+                var projectedPl = projectedPls[0];
+                if (!isNull(projectedPl)) {
+                    projectedPl.simplify(0.01);
+                    ret.push(projectedPl);
+                }
             }
             this.segmentation = true;
 
@@ -459,12 +494,20 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
 
     if (isArcShape(shape)) {
         var self=this;
+        // TODO: use RShape::transformArc
+        // make RShapeTransformation scriptable
+        // override RShapeTransformation::transform
         var ellipse = ShapeAlgorithms.transformArc(shape,
             function(p) {
                 self.project(p);
             }
         );
-        return [ ellipse ];
+        if (ellipse.isValid()) {
+            return [ ellipse ];
+        }
+        else {
+            return [];
+        }
     }
 
     if (isCircleShape(shape)) {
@@ -490,6 +533,7 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             }
         }
 
+        // create polyline from ellipse:
         var ple = ShapeAlgorithms.approximateEllipse(shape, 16);
         return this.projectShape(ple, preview, trim);
     }
@@ -526,6 +570,11 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
     }
 
     if (isPolylineShape(shape)) {
+        if (!rec) {
+            // remove duplicate vertices (zero length segments):
+            shape.normalize(RS.PointTolerance);
+        }
+
         // add polyline, split up at arcs that are projected into ellipses and
         // at gaps caused by trimming:
         ret = [];
@@ -536,7 +585,8 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
         // project each segment individually:
         for (i=0; i<segmentCount; i++) {
             s = shape.getSegmentAt(i).data();
-            s = this.projectShape(s, preview, trim);
+
+            s = this.projectShape(s, preview, trim, true);
 
             // iterate through projected shapes (trimming might lead to gaps):
             for (k=0; k<s.length; k++) {
@@ -558,6 +608,7 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
                 // s is an ellipse:
                 var gotEllipse = isEllipseShape(seg);
 
+                // reached last segment of current polyline:
                 if (i===segmentCount-1 || gotEllipse || gotGap) {
                     if (pl.countVertices()===1) {
                         // don't add polyline for one single shape:
@@ -569,7 +620,10 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
                     }
 
                     if (gotEllipse) {
-                        ret.push(seg);
+                        if (seg.isValid()) {
+                            ret.push(seg);
+                        }
+
                         pl = new RPolyline();
                     }
                     else if (gotGap){
